@@ -5,15 +5,18 @@ Tools to compare semantic versions using npm style version specifiers.
 import importlib.metadata
 import logging
 
+import requests
 from semver import Version
 
 from atopile import errors
 
 log = logging.getLogger(__name__)
-log.setLevel(logging.INFO)
+
+DISTRIBUTION_NAME = "atopile"
+UPGRADE_DOCS_URL = "https://atopile.io/#installation"
 
 
-class VersionMismatchError(errors.AtoError):
+class VersionMismatchError(errors.UserException):
     """
     Raise when the compiler version isn't
     compatible with the project version.
@@ -63,11 +66,28 @@ def get_installed_atopile_version() -> Version:
     """
     Get the installed atopile version
     """
-    ap_version_str = importlib.metadata.version("atopile")
+    ap_version_str = importlib.metadata.version(DISTRIBUTION_NAME)
     semver = parse(ap_version_str)
     return semver
 
 
+def get_latest_atopile_version() -> Version | None:
+    """
+    Get the latest atopile version
+    """
+    try:
+        response = requests.get(
+            f"https://pypi.org/pypi/{DISTRIBUTION_NAME}/json", timeout=0.1
+        )
+        response.raise_for_status()
+        version_str = response.json()["info"]["version"]
+    except (KeyError, requests.exceptions.RequestException):
+        log.debug("Failed to get latest version")
+        return None
+
+    return parse(version_str)
+
+
 def match_compiler_compatability(built_with_version: Version) -> bool:
     """
     Check whether the currently installed compiler is compatible with
@@ -84,21 +104,14 @@ def match_compiler_compatability(built_with_version: Version) -> bool:
         else:
             compiler_semver = compiler_semver.bump_minor()
 
-    # Until the first major release, we're assuming
-    # that minor releases are breaking changes
-    if compiler_semver.major == 0:
-        match_operator = "~"
-    else:
-        match_operator = "^"
-
     # Check if we match
-    return match(f"{match_operator}{built_with_version}", compiler_semver)
+    return match(f">={built_with_version},<0.4.0", compiler_semver)
 
 
-OPERATORS = ("*", "||", "^", "~", "!", "==", ">=", "<=", ">", "<")
+OPERATORS = ("*", "^", "~", "!", "==", ">=", "<=", ">", "<")
 
 
-def match(spec: str, version: Version):
+def match(spec: str, version: Version) -> bool:
     """
     Check if a version matches a given specifier
 
@@ -122,26 +135,24 @@ def match(spec: str, version: Version):
                 return True
         return False
 
-    if " " in spec:
-        for s in spec.split(" "):
+    if "," in spec:
+        for s in spec.split(","):
             if not match(s, version):
                 return False
         return True
 
-    if spec[:2] in ("==", ">=", "<="):
-        operator = spec[:2]
-        specd_version = parse(spec[2:])
-    elif spec[0] in ("^", "~", "!", ">", "<"):
-        operator = spec[0]
-        specd_version = parse(spec[1:])
+    for operator in OPERATORS:
+        if spec.startswith(operator):
+            specd_version = parse(spec[len(operator) :])
+            break
     else:
         # if there's not operator, default to ^ (up to next major)
         try:
             specd_version = parse(spec)
             operator = "^"
         except ValueError as ex:
-            # finally, if we could do any of that, we assume there's something wrong with the spec
-            raise errors.AtoError(f"Invalid version spec: {spec}") from ex
+            # finally, if we could do any of that, we assume there's something wrong with the spec # noqa: E501  # pre-existing
+            raise errors.UserException(f"Invalid version spec: {spec}") from ex
 
     if operator == "^":
         # semver doesn't support ^, so we have to do it ourselves
@@ -170,3 +181,30 @@ def match(spec: str, version: Version):
 
     if operator == "<":
         return version < specd_version
+
+    else:
+        raise ValueError(f"Invalid operator: {operator}")
+
+
+def check_for_update() -> None:
+    installed_version = get_installed_atopile_version()
+    latest_version = get_latest_atopile_version()
+    installed_version_clean = clean_version(installed_version)
+
+    if latest_version is None:
+        return
+
+    if installed_version < latest_version:
+        log.warning(
+            "Your version of atopile (%s) is out-of-date. Latest version: %s.\n"
+            "You can find upgrade instructions here: %s",
+            installed_version_clean,
+            latest_version,
+            UPGRADE_DOCS_URL,
+        )
+    elif installed_version > latest_version:
+        log.info(
+            "Current version (%s) is newer than latest (%s)",
+            installed_version_clean,
+            latest_version,
+        )
